#!/usr/bin/env python
# Copyright 2008 David D Lowe
# released under the GPLv2

from pigment import PigmentType, Pigment
import pigment
import thread
import os, sys
import subprocess
from shell import SubprocessShell
import gconf
from glob import glob
sys.path.append("..")
import ep_exceptions
import handy
import hashlib


class GnomeSplashType(PigmentType):
    humanName = {"en":"Gnome Splash"}
    humanPluralName = {"en":"Gnome Splashes"}
    codeName = "gnomeSplash"
    abbreviation = "spl"
    directoryName = "gnomeSplashes"
        

class GnomeSplash(Pigment):
    
    
    def __init__(self):
        Pigment.__init__(self)
        self.type = GnomeSplashType()
        self.splashFilename = ""
        self.splashPath = ""
    
    def add_attachment(self, attachmentPath):
        if not len(self.attachments) == 0:
           raise(ep_exceptions.TooManyAttachmentsException(len(self.attachments)))
        if os.path.isabs(attachmentPath) and os.path.exists(attachmentPath):
            realPath = attachmentPath
        else:
            if os.path.exists(os.path.join("/usr/share/pixmaps/splash", self.codeName + "-" + attachmentPath.split("/")[-1])):
                realPath = os.path.join("/usr/share/pixmaps/splash", self.codeName + "-" + attachmentPath.split("/")[-1])
            else:
                self.splashPath = ""
                self.splashFilename = ""
                raise(ep_exceptions.AttachmentNotFoundException(attachmentPath))
        Pigment.add_attachment(self, realPath)
        self.splashPath = realPath
        self.splashFilename = attachmentPath.split("/")[-1]
    
    
    def install(self, installationDirectory=None, progressCallback=None,  shell=None):
        exitPlease = False
        if progressCallback is None: progressCallback = lambda progress, query: False
        def change_progress_callback(progress, query):
            if progressCallback(progress / 2, query):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.install(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        # copy gnome splash file
        if progressCallback is None: progressCallback = lambda progress, query: False
        if progressCallback(0.75, None): return
        shell.do(["cp", "-f", self.splashPath, "/usr/share/pixmaps/splash/%s-%s" % (self.codeName, self.splashFilename)])
        shell.do(["chown", "root:root", "/usr/share/pixmaps/splash/%s-%s" % (self.codeName, self.splashFilename)])
        shell.do(["chmod", "644", "/usr/share/pixmaps/splash/%s-%s" % (self.codeName, self.splashFilename)])
        progressCallback(1.0, None)
            
    
    def remove(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        def change_progress_callback(progress):
            if progressCallback(progress / 2):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.remove(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        # remove gnome splash file
        if progressCallback is None: progressCallback = lambda(progress): False
        if progressCallback(0.75): return
        shell.do(["rm", "-r", "/usr/share/pixmaps/splash/%s-%s" % (self.codeName, self.splashFilename)])
        progressCallback(1.0)
    
    def activate(self, progressCallback=None, shell=None):
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if progressCallback(0.0): return
        client = gconf.client_get_default()
        client.set_string("/apps/gnome-session/options/splash_image", \
            os.path.join("/usr/share/pixmaps/splash/", self.codeName + "-" + self.splashFilename))
        client.set_bool("/apps/gnome-session/options/show_splash_screen", True)
        progressCallback(1.0)

    def save(self, filePath, shell=None):
        from glob import glob
        import handy
        if not os.path.exists(self.previewPath) and (not self.preview is None):
            raise(ep_exceptions.PreviewFileNotFoundException(self.previewPath))
        doc = self.information_xml()
        text = doc.toxml()
        
        subprocess.call(["rm", "-f", "-r", "/tmp/epidermis/"])
        subprocess.call(["mkdir", "-p", "/tmp/epidermis/information", "/tmp/epidermis/data/"])
        subprocess.call(["touch", "/tmp/epidermis/information/pigment.xml"])
        fh = open("/tmp/epidermis/information/pigment.xml", "w")
        fh.write(text)
        fh.close()
        
        if shell is None:
            shell = SubprocessShell()
            shell.prepare()
        
        previewFile = self.__give_preview_file()
        shell.do(["cp", previewFile, "/tmp/epidermis/information"])
        if previewFile[0:-len(previewFile.split("/")[-1])] == "/tmp/epidermis":
            shell.do(["rm", previewFile])
        
        ll = [l2.split("/")[-1] for l2 in glob("/tmp/epidermis/*")]
        for item in self.attachments:
            shell.do(["cp", "-r", item, os.path.join("/tmp/epidermis/data/", self.splashFilename)])
        shell.do(handy.join_lists(["tar", "-czf", filePath], ll), cwd="/tmp/epidermis/")
    
    def __give_preview_file(self):
        """Returns the absolute path to a preview file for this pigment.
        If the preview filename hasn't been set, but a Pixbuf has, this
        will save a temporary file named preview.png and return its path"""
        subprocess.call(["mkdir", "/tmp/epidermis"])
        if len(self.previewFilename) > 0 and len(self.previewPath) == 0:
            self.preview.save("/tmp/epidermis/" + previewFileName, previewFileName.split(".")[-1].lower())
            return "/tmp/epidermis/" + previewFileName
        elif len(self.previewFilename) == 0 and not self.preview is None:
            self.preview.save("/tmp/epidermis/preview.png", "png")
            return "/tmp/epidermis/preview.png"
        else:
            return self.previewPath
        
    def read(self, filePath):
        Pigment.read(self, filePath)
        
        from xml.dom import minidom
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]
        
        attachmentPath = filePath[:(-1*len(filePath.split("/")[-1]))]
        self.add_attachment(os.path.join(attachmentPath, "data",pigment.get_first_text( \
            root.getElementsByTagName("filename")[0])))
    
    def information_xml(self):
        """Returns the XML document describing the pigment"""
        doc = Pigment.information_xml(self)
        root = doc.getElementsByTagName("pigment")[0]
        
        
        xmlBit = doc.createElement("filename")
        tn = doc.createTextNode(self.splashFilename)
        xmlBit.appendChild(tn)
        root.appendChild(xmlBit)
        
        return doc
    
    
    def is_installed(self, installationDirectory=None):
        """Returns whether this pigment is installed
        installationDirectory: the directory in which to search for already 
        installed pigments"""
        if Pigment.is_installed(self, installationDirectory):
            if os.path.exists(os.path.join("/usr/share/pixmaps/splash/", self.codeName + "-" + self.splashFilename)):
                return True
        return False
    
    def is_activated(self):
        if self.is_meat_activated():
            # TODO: uu
            client = gconf.engine_get_default()
            return self.splashPath == client.get_string("/apps/gnome-session/options/splash_image")
        else:
            return False
    
    def convert_from_meat(self, meat, progressCallback=None):
        # TODO: convert_from_meat
        pass
    
    def is_meat_installed(self):
        """Returns whether the meat of this pigment or similar to it is installed"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = file(self.splashPath, 'rb')
            f2 = file(os.path.join("/usr/share/pixmaps/splash/", self.codeName + "-" + self.splashFilename), 'rb')
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            ss = ff.read(8096)
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
        
    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        client = gconf.client_get_default()
        if client.get_bool("/apps/gnome-session/options/show_splash_screen", True) is False:
            return False
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = file(self.splashPath, 'rb')
            f2 = file(client.get_string("/apps/gnome-session/options/splash_image"), 'rb')
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            ss = ff.read(8096)
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return False
    
    def match_filter(self, string, lang="en"):
        """Returns whether this pigment should be a result for a search for the
        string passed"""
        for ii in (self.get_human_name(lang), self.get_description(lang)):
            if string.lower() in ii.lower():
                return True
        return False
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return

def main():
    wp = GnomeSplash()
    print wp.information_xml().toxml()

if __name__ == "__main__":
    main()